<?php

define('SKIP_KOHANA', true);
require('/opt/monitor/op5/ninja/index.php');

include_once('include/webconfig.inc.php');
include_once('classes/common_gui.php');
include_once('include/nacoma_access.inc.php');
include_once('include/auth_config.inc.php');
require_once('op5/config.php');
require_once('op5/auth/Auth.php');
require_once('op5/log.php');

$log = Op5Log::instance('auth');

$drivers = array();

validate_user();
$page->html_start('Auth Modules Configuration');

$errors = array();
// Only handles the "submit" action and not ajax calls
if(isset($_REQUEST['action'])) {
	$configMap = array();
	$renameMap = array();
	$newConfig = array('common' => '');
	$oldConfig = Op5Config::instance()->getConfig('auth');
	$hiddenModulesInOldConfig = array();
	foreach($oldConfig as $module => $config) {
		if($module != 'common' && authdriver_disabled_in_gui($config['driver'])) {
			$hiddenModulesInOldConfig[] = $module;
			$newConfig[$module] = $config;
		}
	}

	//Separate driver name from setting key.
	foreach($_REQUEST as $key => $value) {
		if($key == 'csrf_token') {
			// ignore this, for now
			continue;
		}
		$key = explode('-', $key);
		if ($key[0] == 'action' || $key[0] == 'action_dummy') {
			continue;
		}
		$configMap[$key[0]][$key[1]] = $value;
	}
	ksort($configMap);
	foreach ($configMap as $key => $value) {
		if ($key == 'common') {
			$newConfig[$key] = $authmodule_fields['common'];
		} else {
			if(authdriver_disabled_in_gui($value['driver'])) {
				echo "Can not find the driver ".$value['driver'].", please try again";
				$page->html_end(1);
			} elseif(in_array(strtolower($key), array_map('strtolower', $hiddenModulesInOldConfig), true)) {
				$errors[] = "You tried to create a module called '$key' but that's already taken by a module hidden from the GUI (see auth.yml for further information)";
				continue;
			}
			$newConfig[$key] = $authmodule_fields[$value['driver']];
			$newConfig[$key]['driver'] = $value['driver'];
			if($value['rename_module'] == 'common') {
				echo "Can not rename an auth module to 'common', try with another name";
				$page->html_end(1);
			}
			if($value['rename_module'] != $key && array_key_exists($value['rename_module'], $configMap)) {
				printf('You tried to rename "%1$s" to "%2$s" but "%2$s" already exists. Try again.',
					$key, $value['rename_module']);
				$page->html_end(1);
			}
		}
		foreach ($newConfig[$key] as $confKey => $confVal) {
			if ($confKey == 'bind_secret')
			{
				if (substr($_REQUEST[$key.'-'.$confKey], 0, 5) == 'file:') {
					// Bind secret refers to a file containing the secret
					$tmpArray = explode(': ', $_REQUEST[$key.'-'.$confKey]);
					$newConfig[$key][$confKey] = array();
					$newConfig[$key][$confKey]['file'] = trim($tmpArray[1]);
					unset($tempArray);
					continue;
				} elseif (substr($_REQUEST[$key.'-'.$confKey], 0, 7) == 'config:') {
					// Bind secret refers to a config setting containing the secret
					$tmpArray = explode(': ', $_REQUEST[$key.'-'.$confKey]);
					$newConfig[$key][$confKey] = array();
					$newConfig[$key][$confKey]['config'] = trim($tmpArray[1]);
					unset($tempArray);
					continue;
				}
			} else if ($confKey === 'ldap_options') {
				// Special case for LDAP options
				$newConfig[$key][$confKey] = array();
				for ($i=0;$i<count($_REQUEST[$key.'-'.$confKey]);$i++) {
					if (in_array($_REQUEST[$key.'-'.$confKey][$i], $ldap_opts)) {
						$newConfig[$key][$confKey][$_REQUEST[$key.'-'.$confKey][$i]] = $_REQUEST[$key.'-'.$confKey][$i+1];
					}
				}
				continue;
			}
			if (isset($_REQUEST[$key.'-'.$confKey]) && $_REQUEST[$key.'-'.$confKey] == '' && $newConfig[$key][$confKey]['value'] === false) {
				$newConfig[$key][$confKey] = false;
			} elseif (isset($_REQUEST[$key.'-'.$confKey]) && $_REQUEST[$key.'-'.$confKey] !== $confVal) {
				$newConfig[$key][$confKey] = $_REQUEST[$key.'-'.$confKey];
			} elseif (!isset($_REQUEST[$key.'-'.$confKey]) && $newConfig[$key][$confKey]['input'] == 'bool') {
				$newConfig[$key][$confKey] = false;
			} elseif ($confKey !== 'driver') {
				$newConfig[$key][$confKey] = '';
			}
		}
		if (isset($configMap[$key]['rename_module']) && $key !== $configMap[$key]['rename_module'] && strlen($configMap[$key]['rename_module']) > 0) {
			// We're changing the name of the module
			if (preg_match('/[^a-zA-Z0-9]/', $configMap[$key]['rename_module'])) {
				$errors[] = "Driver names can only contain alphanumerical characters";
			} elseif(in_array(strtolower($configMap[$key]['rename_module']), array_map('strtolower', $hiddenModulesInOldConfig), true)) {
				$errors[] = "You tried to rename a module to '".$configMap[$key]['rename_module']."' but that's already taken by a module hidden from the GUI (see auth.yml for further information)";
			} else {
				// Just put the rename in an array for now
				$renameMap[$key] = $configMap[$key]['rename_module'];
			}
		}
		if ($key !== "common") {
			unset($newConfig[$key]['rename_module']);
		}
	}
	if (!$errors) {
		// Write config, disregarding renames if there are any
		Op5Config::instance()->setConfig('auth', $newConfig);
		foreach ($renameMap as $old => $new) {
			// Rename modules as well as all relations
			op5auth::rename_module($old, $new);
		}
		$log->log('debug', get_user() . ' changed the Auth Module Configuration');
	}
}

?>
<div id="dialog" title="New Module">
	<form action="" method="post">
		<fieldset class="ui-helper-reset">
			<label for="driver_name">Module Name</label>
			<input type="text" name="driver_name" id="driver_name" value="" class="module_name ui-widget-content ui-corner-all" autocomplete="off" />
			<label for="driver_type">Driver Type</label>
			<select name="driver_type" id="driver_type" class="ui-widget-content ui-corner-all">
				<option value="Default">Default</option>
				<option value="LDAP">LDAP</option>
				<option value="Apache">Apache</option>
			</select>
		</fieldset>
	</form>
</div>
<?php

$access = get_nacoma_access('access_rights');
if (!in_array(NACOMA_ACCESS_FULL_ACCESS, $access)) {
	deny_access("You need full 'access_rights' permissions to modify Auth Module Configuration.");
}
$authconf = Op5Config::instance()->getConfig('auth');
if ($errors) {
	echo '<div id=errors><p>';
	foreach ($errors as $message) {
		echo $message . '<br />';
	}
	echo '</p></div>';
}
echo "<div id='driver-tabs'>\n";
echo "<ul>\n";
foreach ($authconf as $driver => $driverArray) {
	//We don't want common in the driver list
	if ($driver === 'common') {
		echo "<li><a href='#$driver'>$driver</a></li>\n";
	} else {
		$drivers[] = $driver;
		echo "<li><a href='#$driver'>$driver</a> <span class='ui-icon ui-icon-close' id='delete-$driver'>$driver</span></li>\n";
	}
}
echo "<li><a href='#new_placeholder' id='new_driver'>Add New Module</a></li>\n";
echo "</ul>\n";
$apc_not_installed = "APC is not installed on this machine.";
$page->form_start();
foreach ($authconf as $driver => $driverSettings) {
	echo "<div id='$driver'>\n";
	//Special settings for common.
	if ($driver === 'common') {
		echo "<table class=''>\n";
		foreach ($authmodule_fields['common'] as $key => $value) {
			echo "<tr>\n";
			echo "<td><label for='$driver-$key'>" . ucwords(str_replace('_', ' ', $key)) . "</label></td>\n<td>";
			switch ($value['input']) {
				case 'string': case 'int':
					//Check if apc is installed on the machine and behave accordingly
					if (array_key_exists('apc', $authmodule_fields['common'][$key]) && !$authmodule_fields['common'][$key]['apc']) {
						echo $apc_not_installed;
					} else {
						echo "<input id='$driver-$key' name='$driver-$key' type='text' value='$driverSettings[$key]' />";
					}
					break;
				case 'bool':
					if (array_key_exists('apc', $authmodule_fields['common'][$key]) && !$authmodule_fields['common'][$key]['apc']) {
						echo $apc_not_installed;
					} else {
						$checked = (isset($driverSettings[$key]) && $driverSettings[$key] == $value['value']) ? "checked" : "";
						echo "<input id='$driver-$key' name='$driver-$key' type='checkbox' value='1' $checked />";
					}
					break;
				case 'select':
					echo "<select name='$driver-$key' id ='$driver-$key'>\n";
						foreach ($drivers as $option) {
							$selected = (isset($driverSettings[$key]) && $driverSettings[$key] == $option) ? "selected='selected'" : "";
							echo "<option value='$option' $selected>$option</option>\n";
						}
					echo "</select>\n";
					break;
			}
			echo "</td>\n";
			echo "<td>\n";
			echo " &nbsp; <img src='images/icons/shield-help.png' class='help_icon' style='cursor: pointer;' alt='$key' title='$key' />\n";
			echo "</td>\n";
			echo "</tr>\n";
		}
		echo "</table>\n";

	} else {
		echo driverForm($driver, false, $driverSettings);
	}
	echo "</div>\n";
}
echo "<div id='new_placeholder'></div>";
echo "</div>\n";
echo '<div class="max magic">';
$page->submit_button( "Save", false, false, array( 'id' => 'Save' ) );
echo '</div>';
$page->form_end();
$page->html_end();
