<?php
/*
 * Copyright (C) 2004, 2005, 2007, 2011 op5 AB
 * All rights reserved.
 *
 */

require_once('config.inc.php');
require_once('webconfig.inc.php');
require_once('nacoma_access.inc.php');
require_once('changelog.inc.php');
require_once('op5/log.php');


/**
 * Fetches custom vars for the $type object with id $id
 */
function get_custom_vars($obj_type, $obj_id)
{
	$custom_table = 'custom_vars';
	if ($obj_type === 'management_pack')
		$custom_table = 'persistent_custom_vars';
	$query = "SELECT * FROM $custom_table " .
		'WHERE obj_type = '.nacoma_db::quote_string($obj_type).' AND obj_id = '.nacoma_db::quote_string($obj_id);
	$result = nacoma_db::exec_query($query);
	$ret = array();
	while (($ary = $result->fetch(PDO::FETCH_ASSOC)))
		$ret[$ary['variable']] = $ary['value'];
	unset($result);
	return $ret;
}

/* returns the proper data-structure for the object type in question
 * warning: do not under any condition use for any purpose ever - ask
 * the object class.
 */
function get_object_properties($obj_type) {
	if (class_exists( $obj_type . '_nacoma_Model' )) {
		$class = $obj_type . '_nacoma_Model';
		$obj = new $class();
		return $obj->get_properties();
	}
	if (strpos($obj_type, '_template') !== false) {
		$props =get_object_properties(substr($obj_type, 0, -strlen('_template')));
		if($prop === false)
			return false;
		if(!isset($props['visible'])) {
			$props['visible'] = array_keys($pros['default']);
		}
		if(!in_array('name', $props['visible'])) {
			$props['visible'][] = 'name';
		}
		return $props;
	}
	return false;
}

/**
 * @name	glue_custom_vars
 * @param	object type (string)
 * @param	object id (integer)
 * @param	array (custom variables, k => v style)
 */
function glue_custom_vars($obj_type, $obj_id, $custom)
{
	global $IMPORTING;

	$ret = true;

	# optimize for when we're importing, as we can safely
	# avoid running tens of thousands of queries that way
	if ($IMPORTING && empty($custom))
		return true;

	$esc_obj_type = nacoma_db::quote_string($obj_type);
	$esc_obj_id = nacoma_db::quote_string($obj_id);
	$custom_table = 'custom_vars';
	if ($obj_type === 'management_pack')
		$custom_table = 'persistent_custom_vars';
	$purge_query = 'DELETE FROM '.$custom_table.' WHERE ' .
		"obj_type = $esc_obj_type AND obj_id = $esc_obj_id";

	$result = nacoma_db::exec_query($purge_query);
	if (!$custom)
		return $result;

	$base_query = "INSERT INTO $custom_table VALUES(" .
		$esc_obj_type . "," . $esc_obj_id . ", ";

	foreach ($custom as $k => $v) {
		$query = $base_query . nacoma_db::quote_string($k) . "," .
			nacoma_db::quote_string($v) . ")";
		$result = nacoma_db::exec_query($query);
		if (!$result)
			$ret = false;
		unset($result);
	}

	return $ret;
}

/* takes an indexed array of associative arrays as input and writes them
 * to database. This is the function every script that modifies objects
 * MUST use to glue their objects back together, once they have been
 * normalised. It also handles the junction tables and ensures the
 * database integrity.
 */
function glue_objects($obj_list, $table_name) {
	global $DEBUG;
	global $SUPPORT;
	global $IMPORTING;

	# no objects, so we're done glueing
	if(empty($obj_list)) return true;

	$db_obj_insert = false;
	$Junctions = false;
	$delayed_inserts = false;
	$p = get_object_properties($table_name);
	$v_index = $p['var_index']; // let's recognise checkbox vars properly
	$spec = $p['special'];

	# build the list of junction tables we need to delete references from
	$jnc_list = array();
	if(!$IMPORTING) {
		$jnc_list = get_junction_list($table_name);
		if($table_name === 'host') {
			unset($jnc_list['parents']);
			unset($jnc_list['children']);
		}
	}

	// for debugging purposes
	$db_obj_insert[$table_name] = 0;
	$db_obj_insert['real'][$table_name] = 0;

	# hoist this out of the loop. one hash-lookup is much faster than
	# several if() branches when there are many objects (when there are few
	# it doesn't matter anyway)
	if(isset($obj_list['new'])) {
		$nid = get_next_object_id($table_name);
		$obj_list[$nid] = $obj_list['new'];
		unset($obj_list['new']);
	}

	foreach(array_keys($obj_list) as $obj_key) {
		if ($table_name == 'FILE') {
			nacoma_db::exec_query("DELETE FROM file_tbl WHERE id=$obj_key");
		}
		else {
			nacoma_db::exec_query("DELETE FROM $table_name WHERE id=$obj_key");
		}
	}

	# commands are inherently simple (no out-references, no junctions, no
	# checkboxes), so we handle those separately for speed. They are also
	# always committed in bulk (up to 100 at a time), so it's a reasonable
	# simplicity / performance trade-off
	if($table_name === 'command') {
		$stmt = nacoma_db::prepare_statement("INSERT INTO command VALUES(:id, :name, :line, 1, :fileid)");
		foreach($obj_list as $obj_key => $obj) {
			$stmt->execute(array('id' => $obj_key, 'name' => $obj['command_name'], 'line' => $obj['command_line'], 'fileid' => $obj['file_id']));
		}
		return true;
	}

	# FILE objects are even simpler, so we handle those separately
	# as well. This also avoids the whole issue of FILE being a
	# reserved word in Oracle and other...
	if ($table_name === 'FILE') {
		foreach ($obj_list as $obj_key => $obj) {
			nacoma_db::exec_query("INSERT INTO file_tbl VALUES('$obj_key', " .
						   nacoma_db::quote_string($obj['file_name']) . ")");
		}
		return true;
	}

	# not a command, so do the complex loop
	foreach($obj_list as $obj_key => $obj) {
		# only handle numerical object_keys. We really should print some sort
		# of 'hack-atack' statement here, but the fact that it just doesn't
		# work might be a deterrant as well.
		if(!is_numeric($obj_key)) continue;
		$obj['id'] = $obj_key;

		# FILE is a reserved word in at least oracle, so we must
		# rename it to avoid running into sql errors
		if (isset($obj['FILE'])) {
			$obj['file_id'] = $obj['FILE'];
			unset($obj['FILE']);
		}

		# purge all junction references indiscriminately.
		# They will be re-added later on if they're still there, and this
		# way we save ourselves a couple of cmp/branch
		if($table_name === 'host' && !$IMPORTING) {
			# Note that we can't delete 'the other end' for parents and
			# such, or we would force users to have their hosts configured
			# in reverse order of their network topology (bug #181).
			# This doesn't present a problem so long as we remember
			# to purge it when *deleting* a host
			nacoma_db::exec_query("DELETE FROM host_parents " .
						   "WHERE host = $obj_key");
			# we can if it's either not empty or not set though though,
			# since it will be auto-updated if it is, and only new children
			# can be added (none removed) unless we do.
			if (!empty($obj['children']) || !isset($obj['children']))
			  nacoma_db::exec_query("DELETE FROM host_parents " .
							 "WHERE parents = $obj_key");
		}
		if (!$IMPORTING && !empty($jnc_list) && is_array($jnc_list)) {
			foreach($jnc_list as $k => $v) {
				$jnc = get_junction_table_name($table_name, $k);
				nacoma_db::exec_query("DELETE FROM $jnc WHERE $table_name = $obj_key");
			}
		}

		// loop every variable in the object
		foreach($obj as $k => $v) {
			if(empty($v) && $v !== '0' && $v !== 0) {
				unset($obj[$k]);
			}
			// all arrays that are still around require junctions
			// we know this since we've uniqueified every other object
			elseif (is_array($v)) {
				if($v_index[$k] === 'checkbox') {
					$obj[$k] = stringify_checkbox($v);
					continue;
				}

				// not a checkbox var, so it must be a junction.
				unset($obj[$k]);
				$query = false;
				$junction = get_junction_table_name($table_name, $k);

				# set up the values and insert them
				# All junctions are indiscriminately purged above, so
				# it should be safe to do this now.
				foreach($v as $junc_part) {
					if(empty($junc_part) || $junc_part === '0') {
						echo "SQL WARNING!! Inserting empty values in $junction. type: $table_name, var: $k, key: $obj_key\n";
						echo "This is an index normalisation bug. Please contact $SUPPORT so that this issue may be resolved.<br />\n";
					}
					$values = nacoma_db::quote_string($obj_key) . ', ' . nacoma_db::quote_string($junc_part);
					$other_obj_type = $spec[$k];
					// hosts require special handling
					$query = false;
					if ($k === 'children') {
						$query = "INSERT INTO host_parents " .
						  "(parents, host) VALUES($values)";
					}
					elseif (substr($junction, -13) === 'group_members') {
						$query = "INSERT INTO $junction " .
							"($table_name, members) VALUES($values)";
					}
					elseif ($other_obj_type === $table_name) {
						# self-multi-referencing object.
						# 'timeperiod_exclude' or 'host_parents'
						$other_obj_type = $k;
					}

					if($query === false) {
						if (empty($obj_key)) {
							$delayed_inserts[] = "INSERT INTO $junction " .
							  "(" . $table_name . ',' . $other_obj_type . ') ' .
						   "VALUES(%s, ".nacoma_db::quote_string($junc_part).")";
						} else {
							$query = "INSERT INTO $junction " .
							  "(" . $table_name . ',' . $other_obj_type . ') ' .
						   "VALUES($values)";
						}
					}
					if (!empty($query))
						nacoma_db::exec_query($query);
				}
			}
		}

		// ok... obj has been purged of all evil arrays, and strings are
		// properly escaped. It *should* be safe to insert it into the db now.
		$db_obj_insert[$table_name]++;
		$target_vars = implode(',', array_keys($obj));
		$target_values = ':'.implode(', :', array_keys($obj));
		$query = "INSERT INTO $table_name($target_vars) " .
		  "VALUES($target_values)";
		$stmt = nacoma_db::prepare_statement($query);
		try {
			$result = $stmt->execute($obj);
		} catch (PDOException $ex) {
			nacoma_db::print_error($ex);
		}


		# if we are inserting a new object we don't have the id to use
		# for junction data yet. The queries that should be committed are then
		# stored in $delayed_inserts and can be built with a simple sprintf()
		if (!empty($delayed_inserts)) {
			$new_id = nacoma_db::last_insert_id($table_name);
			if (!empty($new_id)) {
				foreach ($delayed_inserts as $sql) {
					nacoma_db::exec_query(sprintf($sql, $new_id));
				}
			}
		}
		$db_obj_insert['real'][$table_name]++;
		$obj_array[$table_name][$obj_key] = $obj;
	}
	return true;
}

function get_junction_table_name($obj_type, $v_name) {
	if($obj_type . 'groups' === $v_name) {
		return($obj_type . '_' . $obj_type . 'group');
	}
	if($v_name . 'group' === $obj_type) {
		return($v_name . '_' . $v_name . 'group');
	}
	if($v_name === 'members') {
		return(substr($obj_type, 0, -5) . '_' . $obj_type);
	}
	if($v_name === 'contact_groups') {
		return($obj_type . '_contactgroup');
	}
	if ($v_name === 'contacts')
		return $obj_type . '_contact';

	if($v_name === 'children') {
		return('host_parents');
	}
	if(substr($v_name, -13) === 'group_members')
		return $v_name;

	return($obj_type . '_' . $v_name);
}

function get_object_name_by_array($obj_type, $obj) {
	if (!isset($obj['id']) || !$obj['id'])
		$obj['id'] = 'new';

	$friendly_type = str_replace('_t', ' T', ucfirst($obj_type));

	if ($obj['id'] === 'new')
		$friendly_type = 'New ' . $friendly_type;

	if (count($obj) === 1)
		return $friendly_type;

	if (isset($obj['name']) && $obj['name']) {
		return $obj['name'];
	}
	$obj_type = strtolower($obj_type);

	switch ($obj_type) {
	 case 'file':
	 case 'service':
	 case 'service_template':
	 case 'servicegroup':
	 case 'host':
	 case 'host_template':
	 case 'hostgroup':
	 case 'contact':
	 case 'contact_template':
	 case 'contactgroup':
	 case 'timeperiod':
	 case 'command':
	 case 'management_pack':
		if ($obj_type == 'service' && isset($obj['service_description']) && $obj['service_description']) {
			if (isset($obj['host_name']) && $obj['host_name']) {
				$host_name = get_object_name_by_id('host', $obj['host_name']);
				return $host_name.';'.$obj['service_description'];
			}
			else if (isset($obj['hostgroup_name']) && $obj['hostgroup_name']) {
				$hostgroup_name = get_object_name_by_id('hostgroup', $obj['hostgroup_name']);
				return $hostgroup_name.';'.$obj['service_description'];
			}
		}
		else if (isset($obj[$obj_type.'_name']) && $obj[$obj_type.'_name']) {
			return $obj[$obj_type.'_name'];
		}
		else if (isset($obj['name']) && $obj['name']) {
			return $obj['name'];
		}

		return "Unknown " . $friendly_type;
		break;
	 case 'servicedependency':
		if (empty($obj['service']) || empty($obj['dependent_service']))
			$name = $friendly_type;
		else
			$name = '"'.get_object_name_by_id('service', $obj['dependent_service']).'" depends on "'.get_object_name_by_id('service', $obj['service']).'"';
		break;
	 case 'hostdependency':
		$master_type = !empty($obj['dependent_hostgroup_name']) ? 'hostgroup' : 'host';
		$name = '"'.get_object_name_by_id($master_type, $obj['dependent_'.$master_type.'_name']).'"';
		$deps = array();
		$ht = new host_template_nacoma_Model(isset($obj['host_name'])?$obj['host_name']:false);
		if (isset($obj['host_name']) && $obj['host_name'] && $ht->id)
			$deps[] = 'host template "'.get_object_name_by_id('host', $obj['host_name']).'"';
		else if (isset($obj['host_name']) && $obj['host_name'])
			$deps[] = 'host "'.get_object_name_by_id('host', $obj['host_name']).'"';
		if (isset($obj['hostgroup_name']) && $obj['hostgroup_name'])
			$deps[] = 'hostgroup "'.get_object_name_by_id('hostgroup', $obj['hostgroup_name']).'"';
		if (!empty($deps))
			$name .= ' depends on '.implode(' and ', $deps);
		else
			$name = $friendly_type . ' for ' . $name;
		break;
	 case 'serviceescalation':
		$name = 'Service Escalation';
		if (isset($obj['first_notification']) && $obj['first_notification'] && isset($obj['last_notification']) && $obj['last_notification'])
			$name .= ' '.$obj['first_notification'].' - '.$obj['last_notification'];
		if (isset($obj['service']) && $obj['service'])
			$name .= ' for '.get_object_name_by_id('service', $obj['service']);
		else
			$name = 'New ' . $name;
		if ($obj['id'] && $obj['id'] != 'new')
			$name .= ' (#'.$obj['id'].')';
		break;
	 case 'hostescalation':
		$name = 'Host Escalation';
		if (isset($obj['first_notification']) && $obj['first_notification'] && isset($obj['last_notification']) && $obj['last_notification'])
			$name .= ' '.$obj['first_notification'].' - '.$obj['last_notification'];
		if (isset($obj['host_name']) && $obj['host_name'])
			$name .= ' for host ' . get_object_name_by_id('host', $obj['host_name']);
		else if (isset($obj['hostgroup_name']) && $obj['hostgroup_name'])
			$name .= ' for hostgroup ' . get_object_name_by_id('hostgroup', $obj['hostgroup_name']);
		else
			$name = 'New ' . $name;
		if ($obj['id'] && $obj['id'] != 'new')
			$name .= ' (#'.$obj['id'].')';
		break;
	 default:
		if (isset($obj[$obj_type.'_name']) && !empty($obj[$obj_type.'_name']))
			$name = $obj[$obj_type.'_name'];
		else if ($obj['id'] !== 'new')
			$name = $friendly_type.' #'.$obj['id'];
		else
			$name = $friendly_type;
		break;
	}
	return $name;
}

function get_object_name_by_id($obj_type, $obj_id) {
	if ($obj_id < 1) {
		$friendly_type = str_replace('_t', ' T', ucfirst($obj_type));
		return "New " . $friendly_type;
	}
	try {
		$fields = array('id');
		switch ($obj_type) {
		case 'host':
		case 'contact':
			$fields[] = $obj_type . '_name';
			$fields[] = 'name';
			break;
		case 'service':
			$fields[] = 'host_name';
			$fields[] = 'hostgroup_name';
			$fields[] = 'service_description';
			$fields[] = 'name';
			break;
		 case 'servicedependency':
			$fields[] = 'dependent_service';
			$fields[] = 'service';
			break;
		 case 'serviceescalation':
			$fields[] = 'service';
			$fields[] = 'first_notification';
			$fields[] = 'last_notification';
			$fields[] = 'id';
			break;
		 case 'hostdependency':
			$fields[] = 'dependent_host_name';
			$fields[] = 'dependent_hostgroup_name';
			$fields[] = 'host_name';
			$fields[] = 'hostgroup_name';
			break;
		 case 'hostescalation':
			$fields[] = 'host_name';
			$fields[] = 'hostgroup_name';
			$fields[] = 'first_notification';
			$fields[] = 'last_notification';
			$fields[] = 'id';
			break;
		 default:
			if (!is_numeric($obj_id))
				return $obj_id; # Looks presentable enough, donnit?
			$fields[] = $obj_type . '_name';
			break;
		}
		$res = nacoma_db::exec_query("select ".implode(',', $fields)." from $obj_type where id = $obj_id", false);
		$attribs = $res->fetchAll(PDO::FETCH_ASSOC);
		if (!empty($attribs))
			$attribs = $attribs[0];
		else
			$attribs = array();
		unset($res);
	} catch (PDOException $ex) {
		return false;
	}

	return get_object_name_by_array($obj_type, $attribs);
}

function get_junction_list($obj_type) {
	$jnc_ref = false;

	if(is_array($obj_type)) {
		bug("ERROR: get_junction_list(): obj_type is an array!", true);
	}
	$props = get_object_properties($obj_type);
	$spec = $props['special'];
	if(!$props) return(false);
	foreach($props['var_index'] as $k => $v) {
		if($v === 'mselect' && !(array_key_exists($k, $spec) && is_array($spec[$k])))
			$jnc_ref[$k] = $spec[$k];
	}

	return($jnc_ref);
}

function get_junction_data($obj_type, $obj_key, $obj_var, $jnc_var) {
	$jnc_data = false;

	$table = get_junction_table_name($obj_type, $obj_var);

	// deal with self-referencing objects (host_parents, *group_members for now)
	$query = false;
	if($obj_type === $jnc_var)
		$jnc_var = $obj_var;
	if($obj_type === 'host' && $obj_var === 'children') {
		$query = "SELECT host FROM host_parents WHERE parents=" . nacoma_db::quote_string($obj_key);
	} else if ($obj_type === 'hostgroup' && $obj_var === 'members') {
		$query = "SELECT host_hostgroup.host FROM host_hostgroup" .
			" INNER JOIN host on host.id = host_hostgroup.host" .
			" WHERE host_hostgroup.hostgroup=" . nacoma_db::quote_string($obj_key) .
			" AND host.host_name IS NOT NULL";
	} else if (substr($obj_var, -13) === 'group_members') {
		$query = "SELECT members FROM $obj_var WHERE $obj_type=".nacoma_db::quote_string($obj_key);
	}
	if($query === false) {
		$query = "SELECT $jnc_var FROM $table WHERE $obj_type =" . nacoma_db::quote_string($obj_key);
	}
	$result = nacoma_db::exec_query($query);
	while($row = $result->fetch(PDO::FETCH_NUM)) {
		$jnc_data[] = (int)$row[0];
	}
	return $jnc_data;
}


function get_object_reference_list($obj_type) {
	$ret = false;
	if(is_array($obj_type)) {
		bug("ERROR: " . __FUNCTION__ . "(): obj_type is an array!", true);
	}
	$p = get_object_properties($obj_type);
	$spec = $p['special'];
	foreach($p['var_index'] as $k => $v) {
		if($v === 'select' || $v === 'mselect') {
			if(!array_key_exists($k, $spec)) continue;
			// extinfo objects have images in arrays
			// This probably affects host/service objects, since extinfo objects
			// isn't available anymore.
			if(is_array($spec[$k])) continue;
			$ret[$k] = $spec[$k];
		}
	}
	return($ret);
}

function get_object_reverse_reference_list($master_obj_type) {
	global $Supported_Object_Types;

	$ret = false;
	foreach($Supported_Object_Types as $obj_type) {
		$p = get_object_properties($obj_type);
		if(!isset($p['special'])) continue;
		$spec = $p['special'];
		foreach($spec as $k => $v) {
			if($v === $master_obj_type) $ret[$obj_type][] = $k;
		}
	}

	// update cache
	return($ret);
}

// WARNING: you should never call this function unless you're just about to
// push stuff to the DB, as that will cause (more) collisions and crashes.
//
// this is a clumsy function to get highest possible id of an object.
// It should be left to the database instead, but then we'd get objects with
// ridiculously high ID's, which might cause DOS eventually
// OTOH, this is not concurrency-safe.
function get_next_object_id($obj_type) {
	$result = nacoma_db::exec_query("SELECT id FROM $obj_type ORDER BY id DESC LIMIT 1");
	$row = $result->fetch(PDO::FETCH_NUM);
        unset($result);
	return $row ? ($row[0] + 1) : 1;
}

function get_num_hosts_configured() {
	$query = "SELECT count(host_name) AS num_hosts FROM host";
	$result = nacoma_db::exec_query($query);
	$row = $result->fetch(PDO::FETCH_NUM);
	unset($result);
	if($row)
	  return (int) $row[0];
	return 0;
}

function get_num_services_configured() {
	$query = "SELECT count(*) AS num_services FROM service";
	$result = nacoma_db::exec_query($query);
	$row = $result->fetch(PDO::FETCH_NUM);
	unset($result);
	if($row)
	  return (int) $row[0];
	return 0;
}
