<?php
/*
 * Copyright(C) 2003-2005,2007 op5 AB
 * All rights reserved.
 *
 */

/*
 * These are layer 1 functions. They do exactly as they're told, without any
 * kind of conversion or anything. They don't need any include files at all,
 * and are the *LEAST* likely to contain bugs if anything goes amiss...
 */

/****************************************************************************/
/*************************** SYSTEM FUNCTIONS *******************************/
/***********************(the dangerous ones ...) ***************************/
/****************************************************************************/

/* executes a command through exec and returns its output and return value */
function exec_cmd($cmd) {
	$val = false;
	$buf = false;

	exec($cmd . ' 2>&1', $buf, $val);

	$buf = implode("\n", $buf);

	$ret['value'] = $val;
	$ret['buf'] = $buf;

	return($ret);
}

/**
 * Probe a bunch of different stuff by execute the command corresponding
 * to the given "module".
 *
 * @param $module   string       Which "module" to be executed. Each module
 *                               correspond to a specific command string.
 * @param $options  string|array Options for that particular module, which is
 *                               impossible to know without looking below in
 *                               this function.
 * @param $testmode boolean      If true the command is prefixed with echo which
 *                               makes this function return the command string
 *                               that otherwise would be executed.
 * @return Array with key "value" containing the return value from the
 *         executed command, and key "buf" with the output.
 */
function probe($module, $options = false, $testmode = false) {
	$poller = false;
	$node = '';
	$ret = array();
	if (!empty($GLOBALS['poller']) && is_string($GLOBALS['poller'])) {
		$poller = escapeshellarg($GLOBALS['poller']);
		$node = "/usr/bin/asmonitor -q '/usr/bin/mon' 'node' 'ctrl' $poller '--' ";
	}

	// Shell escape options.
	if (is_string($options)) {
		$options = escapeshellarg($options);
	} else if (is_array($options)) {
		// Walk the array structure and shell escape every string.
		array_walk_recursive($options, function(&$v, $k) {
		if (is_string($v))
			$v = escapeshellarg($v);
		});
	}

	switch ($module) {
		case "GET_MERLIN_NODES":
			$cmd = "/usr/bin/asmonitor /usr/bin/mon node show";
			break;
		case "DETECT_NCP":
			$cmd = "$node/opt/plugins/check_nt -H " .
				$options . " -p 1248 -v CLIENTVERSION";
			break;
		case "DETECT_NRPE":
			$cmd = "$node/opt/plugins/check_nrpe -H $options -p 5666";
			break;
		case "DETECT_NOVELL":
			$cmd = "$node/opt/plugins/check_nwstat -H " .
				$options . " -p 9999 -v NLM:MRTGEXT.NLM";
			break;
		case "DETECT_E4":
			$cmd = "$node/opt/plugins/check_http -H " .
				$options . " -u /temp -e Probe";
			break;
		case "DETECT_EM1":
			$cmd = "$node/opt/plugins/check_http -H " .
				$options . " -u /data -e C";
			break;
		case 'DETECT_WMI':
			$cmd = "$node/opt/plugins/check_wmi_plus " .
				"-u {$options['username']} " .
				"-p {$options['password']} " .
				"-H {$options['address']} " .
				"-m info " .
				"-s os";
			break;
		case "PING_SWEEP":
			$cmd = "$node/usr/bin/nmap -sP -T4 -PE --send-ip -R " .
				"-oX - " . implode(' ', $options);
			break;
		case "NMAP_SCAN":
			$tcp_ports = '20-25,53,80,110,119,135,143,220,443,993,1248,3306,5432,5666,6556,9999';
			$cmd = "$node/usr/bin/nmap -T Insane -P0 -sT " .
				"-p $tcp_ports -oX - $options";
			break;
		case "WIN_PARTITION_SCAN":
			$cmd = "$node/opt/plugins/check_nrpe " .
				"-H $options " .
				"-p 5666 " .
				"-c USEDDISKSPACE";
			break;
		case 'WMI_PARTITION_SCAN':
			// do not remove "obviously" redundant quotes - asmonitor and ssh teams up and eats them
			$cmd = "$node/usr/bin/wmic " .
				"-U {$options['username']}%{$options['password']} " .
				"//{$options['address']} " .
				"\"\\\"Select DeviceID from Win32_LogicalDisk where " .
				"Description like '%fixed%'\\\"\"";
			break;
		case 'WMI_NETIF_SCAN':
			$cmd = "$node/usr/bin/wmic " .
				"-U {$options['username']}%{$options['password']} " .
				"//{$options['address']} " .
				"\"\\\"Select Name from Win32_NetworkAdapter where " .
				"AdapterType like '%Ethernet%'\\\"\"";
			break;
		case 'WIN_SERVICE_SCAN':
			if (empty($options['start']))
				$options['start'] = 0;

			$end = '';
			if ($options['arguments_allowed'])
				$end = " -a 'start:{$options['start']}'";
			else if ($options['start'])
				# use script aliases (i.e check_vbs_services20, check_vbs_services40 etc)
				$end = $options['start'];

			$cmd = "$node/opt/plugins/check_nrpe " .
				"-H {$options['address']} " .
				"-c check_vbs_services" . $end;
			break;
		case 'WMI_SERVICE_SCAN':
			$cmd = "$node/usr/bin/wmic " .
				"-U {$options['username']}%{$options['password']} " .
				"//{$options['address']} " .
				"\"\\\"Select DisplayName from Win32_Service\\\"\"";
			break;
		case "WIN_HOST_HAS_PARTITION":
			$cmd = "$node/opt/plugins/check_nt -H " . $options['address'] .
				" -v USEDDISKSPACE -l " . $options['partition'];
			break;
		case "SNMP_SCAN":
			if (!$poller) {
				throw new Exception(
					"You shouldn't be calling SNMP_SCAN for local scanning"
				);
			}
			$cmd = "$node/opt/monitor/op5/nacoma/snmp_runner.php " .
				$options['function'] . ' ' . implode(' ', $options['args']);
			break;
		default:
			$ret['value'] = 1;
			$ret['buf'] = 'Incorrect module given to probe.';
			return $ret;
	}

	$val = false;
	$buf = false;
	if ($testmode) {
		exec('echo ' . $cmd, $buf, $val);
	} else {
		op5log::instance('nacoma')->log('debug', $cmd);
		if ($poller) {
			exec($cmd, $buf);
			$val = 0;
			$pattern = "/.*exited with return code ([0-9]+)/";
			foreach ($buf as $line) {
				if (preg_match($pattern, $line, $match)) {
					$val = $match[1];
				}
			}
			$buf = implode("\n", $buf);
			if (($pos = strpos($buf, '#--- REMOTE OUTPUT START ---')) !== false) {
				$buf = substr($buf, $pos + 29);
				$buf = substr($buf, 0, strpos($buf, '#--- REMOTE OUTPUT DONE ----'));
			}
		} else {
			exec($cmd, $buf, $val);
			$buf = implode("\n", $buf);
		}
	}

	$ret['value'] = $val;
	$ret['buf'] = $buf;
	return $ret;
}

/****************************************************************************/
/**************************** FILE FUNCTIONS ********************************/
/****************************************************************************/

/* makes a blahblah. ;) */
function file_error($mode, $name = 'some file') {
	if(!strlen($name)) {
		bug('file_error() called without a filename',
			true, "mode is $mode");
	}
	printf("Encountered problems while handling file %s\n", $name);
	if($mode == 'noexist') {
		printf("The file doesn't appear to exist.");
	}
	else if($mode == 'lock') {
		printf("Unable to obtain lock on file.");
	}
	else printf("Unable to open file for %s.",
				($mode == 'write') ? "writing" : "reading");

	die("Failed to access file.\n");
}

// reads a file and returns it contents
function read_file($name) {
	return(file_get_contents($name));
}

// opens a file, gets an exclusive lock, closes it and returns true on success
function write_file($name, $buf) {
	$cnt = 3;

	if(empty($name)) echo "write_file called with empty filename. supposed to write <br><pre>$buf</pre>\n";
	if(!($fh = fopen($name, 'w'))) file_error('write', $name);
	while($cnt > 0) {
		$cnt--;
		$locked = flock($fh, LOCK_EX | 'NB');
		if(!$locked) sleep(1);
	}
	if(!$locked) file_error('lock', $name);

	fwrite($fh, $buf);
	fflush($fh);
	flock($fh, LOCK_UN);
	fclose($fh);

	return(true);
}
