<?php
/*
 * Copyright(C) 2003-2007 op5 AB
 * All rights reserved.
 *
 */

require_once(dirname(__FILE__).'/../classes/common_gui.php');
require_once('config.inc.php');
require_once('system.inc.php');
require_once('data.inc.php');
require_once('op5/log.php');

/****************************************************************************/
/*************************** ACTION FUNCTIONS *******************************/
/****************************************************************************/

/* makes a comma-separated string of $arr's association keys */
function stringify_checkbox($arr) {
	$str = '';

	if(!is_array($arr)) return($arr);

	/* handle 'n'(for never) option for various variables */
	if(isset($arr['n'])) {
		if(count($arr) === 1)
			return('n');
		else
			unset($arr['n']);
	}
	sort($arr);

	$str = implode(',', $arr);

	return($str);
}

/****************************************************************************/
/**************************** MISC FUNCTIONS ********************************/
/****************************************************************************/
/*
 * 'The other ones'
 */

# these two should be replaced with something fancier later
function deny_access($msg) {
	global $page;
	if ($page)
		$page->html_start(); // noop if it's already been called, so harmless
	echo '<img src="images/icons/shield-critical.png" alt="" style="display: inline; float: left; margin-right: 10px;" />';
	echo "<h2 style=\"margin-top: 4px;float: left\">Access denied!</h2><p style=\"clear:both\">$msg</p>\n";
	if ($page)
		$page->html_end();
	else
		exit(1);
}

function get_rights() {
	$user = Op5Auth::instance()->get_user();
	if (!$user) return array();
	return $user->get_auth_data();
}

function validate_user() {
	if(php_sapi_name() != 'cli') {
		// Make sure session is loaded if no cli/tests
		// Workaround before fully migrated to ninja
		Session::instance();
	}
	return Op5Auth::instance()->authorized_for("configuration_information");
}

/* sets up the playground. All pages get the same generic look. */
function html_start() {
	global $page, $DEBUG;
	$page->html_start();
	if ($DEBUG)
	  bug("OLD html_start() USED");
}

function html_end($exit_code=0) {
	global $page, $DEBUG;
	if ($DEBUG)
	  bug("OLD html_end() USED");
	$page->html_end($exit_code);
}

/**
 * Tell the user he/she has hit a bug and, optionally, bail out
 * @param $msg The message printed to user
 * @param $crash Print debug backtrace and exit or not?
 * @param $debug_msg Message to, hopefully, include in bug-reports
 */
function bug($msg, $crash = false, $debug_msg = false, $backtrace=false) {
	global $SUPPORT;
	global $DEBUG;

	if (!$backtrace)
		$backtrace = debug_backtrace();

	if (PHP_SAPI == 'cli') {
		$br = '<br />';
		$b_start = '<b>';
		$b_end = '</b>';
		$p_start = '<p>';
		$p_end = '</p>';
		$pre_start = '<pre>';
		$pre_end = '</pre>';
	} else {
		$br = "\n";
		$b_end = $b_start = '';
		$p_end = $p_start = '';
		$pre_start = $pre_end = '';

		if(!headers_sent()) {
			header("HTTP/1.0 500 Internal Server Error");
		}
	}

	$num_args = func_num_args();
	echo "${br}A bug was encountered in $_SERVER[PHP_SELF]$br\n";
	echo "Please contact $SUPPORT regarding this matter.$br$br\n";
	echo "${b_start}${msg}${b_end}${br}${br}\n";

	if($DEBUG && $debug_msg) echo "$debug_msg${br}${br}\n";

	if($crash) {
		if ($DEBUG) {
			echo "Backtrace follows${pre_start}\n";
			print_r($backtrace);
			echo "${pre_end}\n";
		}
		else {
			@mkdir('/tmp/nacoma-stacktraces/', 0700, true);
			$file = tempnam('/tmp/nacoma-stacktraces', date('Ymd-hi').'-');
			$fd = fopen($file, 'w');
			fwrite($fd, $msg."\n");
			fwrite($fd, $debug_msg."\n");
			fwrite($fd, var_export($backtrace, true));

			fclose($fd);

			echo "${p_start}The debug information in ".$file." will be essential to troubleshooting the problem, so please include it if you file a bug report or contact op5 Support.$p_end\n";
		}
		html_end(1);
	}
}

/**
 * Checks if this is a poller system
 * in a distributed setup or not.
 *
 * @return boolean
 */
function is_poller() {
	$mon_cmd=exec_cmd("(/usr/bin/mon node show --type=master|/bin/grep -c 'TYPE=master') 2> /dev/null");
	return $mon_cmd['buf'] > 0;
}

function is_poller_hostgroup($hg_name) {
    $mon_cmd=exec_cmd("(/usr/bin/mon node show --type=poller|/bin/grep -c 'HOSTGROUP=$hg_name') 2> /dev/null");
    return $mon_cmd['buf'] > 0;
}

/**
 * Custom parse_ini_file since default do not handle strings with whitespaces or '='
 *
 * Found in metadata.php
 */
function my_parse_ini_file($metafile){
	$handle = fopen($metafile, 'r');
	$result_array = Array();
	$section = "";
	$plugin = basename(preg_replace('/.metadata/',"",$metafile));
	$result_array['plugin']=$plugin;
	if ($handle) {
		$i=0;
		while (!feof($handle)) {
			$i++;
			$line = fgets($handle);
			/* Strip comments (lines starting with # or ; */
			if(preg_match("/^[#;]/", $line)){
				continue;
			}
			/* Strip blank lines */
			if(trim($line) === ""){
				continue;
			}
			/* Look for section header, enclosed in [] */
			if(preg_match("/^[\[][a-zA-Z0-9]*[\]][ \t]*$/", $line)){
				$section = trim($line, " \t[]\n");
				continue;
			}
			/* This must be a 'section member', store in array  */
			$cmd_name = strtok($line, "=");
			$cmd_line = strtok("");
			if($cmd_line != "")
			  $result_array[$section][$cmd_name] = trim($cmd_line);
			else
			  $result_array[$section][] = trim($cmd_name);
		}
		fclose($handle);
	}
	return($result_array);
}
