(function(window, $) {
"use strict";

function HelpClass()
{
	var that = this;
	$.getJSON(
		'js/helptexts.json',
		function (data) {
			that.helptexts = data;
		}
	);
}
var authconfigHelper = new HelpClass();

function initiateHelptexts(newTab) {
	var selector;
	if (newTab === false) {
		selector = $('.help_icon');
	} else {
		selector = $('#' + newTab + ' .help_icon');
	}
	selector.qtip({
		content: {
			title: {
				text: function() {
					//get the text title attribute for tooltip
					return $(this).closest('tr').find('label').text();
				}
			},
			text: function() {
				var lcheading = $(this).attr('title').toLowerCase();
				if (authconfigHelper.helptexts[lcheading]) {
					return authconfigHelper.helptexts[lcheading];
				}
				return 'No help text available';
			}
		},
		position: {
			my: 'top left',
			at: 'bottom right',
			target: 'event',
			//Confine tooltip to window
			viewport: $(window)
		},
		show: {
			//Show when clicked
			event: 'click'
		},
		style: {
			classes: 'ui-tooltip-dark ui-tooltip-rounded'
		}
	});
}

$(document).ready(function() {
	function validate_chars(driver_name) {
		if (driver_name.match(/[^a-zA-Z0-9]/)) {
			window.alert("ERROR! Invalid character. \nModule name can only contain alphanumerical characters.");
			return false;
		}
		return true;
	}

	var $driver_name_input = $( "#driver_name"),
		$driver_type_input = $( "#driver_type" );

	$('body').on('change', '.module_name', function() {
		validate_chars(this.value);
	});

	// tabs init with a custom tab template and an "add" callback filling in the content
	var $tabs = $( "#driver-tabs").tabs({
		tabTemplate: "<li><a href='#{href}'>#{label}</a> <span class='ui-icon ui-icon-close' id='delete-#{label}'>#{label}</span></li>",
		add: function( event, ui ) {
			var driver_type = $driver_type_input.val() || "Tab " + driver_type + " content.";
			var driver_name = $driver_name_input.val() || "Tab " + driver_name;

			$.ajax({
				type: 'GET',
				url: 'include/auth_config.inc.php?action=newAuthDriverForm&name=' + driver_name + '&type=' + driver_type,
				dataType: 'html',
				success: function(data) {
					$( ui.panel ).append(data);
					initiateHelptexts(driver_name);
					$("#" + driver_name).find("p>select:first").trigger('change');
				},
				error: function(jqXHR) {
					if(jqXHR.responseText) {
						window.alert(jqXHR.responseText);
						return;
					}
					window.alert("Could not save, please try again");
				}
			});
			$tabs.tabs('select', '#' + driver_name);
		}
	});

	function validate_driver_name(driver_name) {
		if (!validate_chars(driver_name)) {
			return false;
		}
		var tabNames = [];
		//Push tab names to array for comparison
		$( "#driver-tabs li" ).each( function() {
			if ($(this).text() !== 'Add New Module') {
				var name = $(this).text().split(" ");
				tabNames.push(name[0]);
			}
		});
		//Cannot create driver with the same name as another tab
		if ($.inArray(driver_name.toLowerCase(), $.map(tabNames, function(name) { return name.toLowerCase(); })) > -1) {
			window.alert("ERROR! Can't create module. \nYou may not use the same module name twice.");
			return false;
		//Cannot create driver with empty name
		} else if (driver_name.indexOf(" ") > -1 || driver_name.length === 0) {
			window.alert("ERROR! Can't create module. \nModule name cannot be empty and cannot contain spaces.");
			return false;
		}
		return true;
	}

	// modal dialog init: custom buttons and a "close" callback reseting the form inside
	var $dialog = $( "#dialog" ).dialog({
		autoOpen: false,
		modal: true,
		buttons: {
			Add: function() {
				if (!validate_driver_name($driver_name_input.val())) {
					return false;
				}
				addTab();
				$( this ).dialog( "close" );
			},
			Cancel: function() {
				$( this ).dialog( "close" );
				$tabs.tabs('select', '#common');
			}
		},
		open: function() {
			$driver_name_input.focus();
			$driver_name_input.keypress(function(e){
				if (e.keyCode === 13){
					e.preventDefault();
				}
			});
		},
		close: function() {
			$form[ 0 ].reset();
		}
	});

	// addTab form: calls addTab function on submit and closes the dialog
	var $form = $( "form", $dialog ).submit(function() {

	});

	// actual addTab function: adds new tab using the title input from the form above
	function addTab() {
		var driver_name = $driver_name_input.val() || "Tab " + driver_name;
		//We need to know how many tabs there are.
		var nextIndex = $( "#driver-tabs li" ).length;
		//Insert tab before add new driver tab.
		$tabs.tabs( "add", "#" + driver_name, driver_name, nextIndex - 1 );
	}

	// addTab button: just opens the dialog
	$( "#new_driver" )
		.click(function() {
			$dialog.dialog( "open" );
		});

	// close icon: removing the tab on click
	// note: closable tabs gonna be an option in the future - see http://dev.jqueryui.com/ticket/3924
	$( "#driver-tabs span.ui-icon-close" ).live( "click", function() {
		var deleteName = $(this).text();
		if (deleteName === 'common') {
			window.alert("The common config can't be deleted!");
		} else if ($( "#driver-tabs li" ).length === 3){
			window.alert("Error! Can't delete module. \nYou need to have at least one module configured.");
		} else if (window.confirm("Do you really want to remove the module '"+deleteName+"'?")) {
			var index = $( "li", $tabs ).index( $( this ).parent() );
			$tabs.tabs( "remove", index );
			$tabs.tabs('select', '#common');
			$('#common-default_auth').removeOption(deleteName);
		}
	});

	var ldap_opts = [
		'LDAP_OPT_DEBUG_LEVEL',
		'LDAP_OPT_DEREF',
		'LDAP_OPT_ERROR_NUMBER',
		'LDAP_OPT_ERROR_STRING',
		'LDAP_OPT_HOST_NAME',
		'LDAP_OPT_MATCHED_DN',
		'LDAP_OPT_NETWORK_TIMEOUT',
		'LDAP_OPT_PROTOCOL_VERSION',
		'LDAP_OPT_REFERRALS',
		'LDAP_OPT_RESTART',
		'LDAP_OPT_SIZELIMIT',
		'LDAP_OPT_TIMELIMIT'
	];

	$("form").on("click", ".add_option", function() {
		var taken = $(this).closest('td').find('select').map( function() {
			return $(this).val();
		}).get();
		var options = $.grep(ldap_opts, function(v) {
			return $.inArray(v, taken) > -1;
		}, true);
		if (!options.length) {
			window.alert("There are no more valid options to add.");
			return;
		}
		var driver = $(this).data('driver');
		var p = $('<p/>').css('width', 'auto');
		var select = $('<select/>').attr('name', driver+'-ldap_options[]');
		var optgroup = $('<optgroup/>').attr('label', 'SELECT LDAP OPTION AND SET VALUE');
		select.append(optgroup);
		$.each(options, function(i, value) {
			var option = $('<option />')
				.text(value)
				.val(value);
			optgroup.append(option);
		});
		var input = $('<input/>').attr('type', 'text').attr('size', '6').attr('name', driver+'-ldap_options[]');
		p.insertBefore($(this));
		p.append(select).append('&nbsp;');
		p.append(input).append('&nbsp;');
		var minus_icon = $('<span/>').addClass('ui-icon ui-icon-circle-minus delete_option').css('display', 'inline-block').css('cursor', 'pointer');
		p.append(minus_icon);
		select.trigger('change');
	});

	$("form").on("click", ".delete_option", function() {
		$(this).closest('p').remove();
		$("table").find("p>select:first").trigger('change');
	});

	$("form").on("change", "p>select", function() {
		var selects = $(this).closest('td').find('select');
		var taken = selects.map( function() {
			return $(this).val();
		});
		var options = $.grep(ldap_opts, function(v) {
			return $.inArray(v, taken) > -1;
		}, true);
		var width = 228;
		selects.each( function() {
			if ($(this).width() > width) {
				width = $(this).width();
			}
			$(this).find('option').not(':selected').remove();
			var optgroup = $(this).children('optgroup');
			$.each(options, function(i, value) {
			var option = $('<option />')
				.text(value)
				.val(value);
			optgroup.append(option);
			});
		});
		selects.width(width);
	});

	initiateHelptexts(false);
	$("table").find("p>select:first").trigger('change');
});
})(window, jQuery);
