<?php

define('SKIP_KOHANA', true);
require('/opt/monitor/op5/ninja/index.php');

require_once('include/webconfig.inc.php');
require_once('include/config.inc.php');
require_once('classes/common_gui.php');
require_once('include/data.inc.php');

$page->html_start("Check Command Import");

nacoma_objlock::die_if_locked_hard();

if (!in_array(NACOMA_ACCESS_EDIT_ALL, get_nacoma_access('command'))) {
	deny_access("You are not allowed to use this feature because you haven't got access to edit commands.");
}

echo "This tool imports factory-default checkcommands into the current configuration. <br />";
echo "The first list below show checkcommands which are missing in the running configuration.<br />";
echo "The second list show checkcommands that differ between current configuration and factory defaults. ";
echo "This is perfectly ok if your check-commands have been modified to better suit your needs. If commands listed ";
echo "have not been manually modified the factory defaults have changed.";

echo "</p></div>";

$page->print_related('check_command_import', array(
	'<img src="images/icons/commands.png" alt="" /> <a href="edit.php?obj_type=command">Commands</a>',
	'<img src="images/icons/manage-management-packs.png" alt="" /> <a href="mgmtpackimport.php">Manage management packs</a>',
));

echo "<div id=\"object-table\">\n";
$page->form_start('metadata', 'post');

$metadata_commands = Array();
$webconfig_commands = Array();
$plugin_info = Array();
$command_group_r = array();

/* Find the metadata files and fetch content    */
$metadata_files = glob("/opt/plugins/metadata/*.metadata");
foreach($metadata_files as $meta_data_file){
	/* Parse with sections using custom function */
	$meta_array = my_parse_ini_file($meta_data_file);
	if(isset($meta_array['commands'])) {
		foreach($meta_array['commands'] as $cmd => $val){
			$metadata_commands["$cmd"] = $val;
			$command_group_r[$cmd] = basename($meta_data_file, '.metadata');
		}
	}
	$plugin = $meta_array['plugin'];

	if(isset($meta_array['info'] )){
		$plugin_info[$plugin] = "";
		foreach($meta_array['info'] as $metainfo){
			$plugin_info[$plugin] .= $metainfo . "<br/>";
		}
		$plugin_info[$plugin] = $plugin_info[$plugin];
	}
	$meta_string = file_get_contents($meta_data_file);
}


/* Insert new commands if posted */
if(isset($_REQUEST['new_command'])){
	$command = new command_nacoma_Model();
	$command->get_default_object();

	/* Loop posted command names, double check if existing, insert if non existing, display warning if not */
	foreach($_REQUEST['new_command'] as $new_cmd => $val){
		$result = nacoma_db::exec_query("SELECT COUNT(*) FROM command WHERE command_name=".nacoma_db::quote_string($new_cmd));
		if($result->fetchColumn(0)) {
			echo "Command '$new_cmd' allready exists. If the command line differ from configured it will be found in the 'modified check commands' section below.<br />";
		} else {
			unset($result);
			if (!$command->obj['file_id']) {
				echo "ERROR: Failed to find checkcommand file. Import aborted<br />";
			} else {
				$command->id = 'new';
				unset($command->obj['id']);
				$command->obj['command_name'] = $new_cmd;
				$command->obj['command_line'] = $metadata_commands[$new_cmd];
				$command->save_object();
			}
		}
	}
}
/* Update existing commands if posted */
if(isset($_REQUEST['modified_command'])){
	foreach($_REQUEST['modified_command'] as $mod_cmd => $val){
		$command = new command_nacoma_Model($mod_cmd);
		$command->obj['command_line'] = $metadata_commands[$mod_cmd];
		$command->save_object();
	}
}

/* Fetch existing commands from webconfig */
$result = nacoma_db::exec_query('SELECT command_name,command_line FROM command');
while($row = $result->fetch(PDO::FETCH_ASSOC)) {
	$webconfig_commands[$row['command_name']] = $row['command_line'];
}

/* New commands. Where command_name is new. */
$new_commands=array_diff_key($metadata_commands, $webconfig_commands);

/* Modified commands. Where command_name exists but commandline differ */
$modified_commands=array_diff_assoc($metadata_commands, $webconfig_commands, $new_commands);

/* List new commands */
$import_avail = FALSE;
echo "<table class=\"max\">
	   <tbody><tr>
	   <td width='100%' class=\"CmdBar\">Plugins with new check commands available </td> <td class=\"CmdBarItem\">
	   <a class='toggle_all' data-commands='new' href='#'>Select&nbsp;All</a></td></tr>
	   </tbody>\n</table>\n";
/* Create the plugin/checkcommand table */
mkTbl($new_commands, "new");

/* List modified commands */
echo "  <table class=\"max\">
	   <tbody><tr>
	   <td width='100%' class=\"CmdBar\">Plugins with modified check commands available </td><td class=\"CmdBarItem\">
	   <a class='toggle_all' data-commands='modified' href='#'>Select&nbsp;All</a></td></tr>
	   </tbody>\n</table>\n";
/* Create the plugin/checkcommand table */
mkTbl($modified_commands, "modified");

function mkTbl($cmds, $prefix){
	global $import_avail;
	global $plugin_info;
	global $webconfig_commands;
	global $command_group_r;
	echo "<table class=\"max zebra\" id='commands_$prefix' style='border-bottom: 1px solid #cdcddc'>";
	$plug="";
	if(!$cmds){
		echo "<tr><td> Configuration up to date </td></tr></table><br />";
		return ;
	}
	foreach($cmds as $cmd => $val) {
		$import_avail = TRUE;
		$thisplug=$command_group_r[$cmd];
		if($plug != $thisplug){
			if($plug!=""){
				echo "</table></div>\n\n";
				echo "</td></tr>";
			}
			$plug = $thisplug;

			echo "\n<tr>";

			echo "<td class='HelpList'>";
			$helptext = "";
			if(isset($plugin_info[$thisplug])) {
				$helptext = $plugin_info[$thisplug];
			}
			echo "<a href=\"#\" title=\"Click to view help\" class='inline_help' data-title='$plug' data-content='$helptext'><img src=\"images/icons/shield-help.png\" alt='' /></a>";
			echo "</td>";
			echo "<td width='99%' style='border-right: 1px solid #cdcddc'>";
			echo "<b><a class='toggle_metadata_desc' href='#'><span>+</span> $plug </a></b></td>\n";
			echo "</tr>";
			echo "\n<tr class=\"meta\"><td colspan='3' style=\"padding: 0px; border: 0px\">";
			echo "\n\n<div style=\"display:none; padding: 4px 0px; border: 1px solid #cdcddc;border-bottom: 1px solid #ffffff;border-top: 0px solid #ffffff;\">";
			echo "\n<table width='100%' style=\"border-collapse: collapse; border-spacing: 0px;\" class=\"zebra\">";

		}
		/* The list of modified commands should include both the new and the existing one */
		if($prefix === 'modified'){
			echo "<tr>";
			echo "<td rowspan='2' style=\"padding-left: 38px; width: 10px;\">";
			echo "<input type='checkbox' name='" . $prefix ."_command[$cmd]' class=\"$prefix\" id=\"{$prefix}_command[$cmd]\"/>";
			echo " </td>";
			echo "<td style=\"padding-left: 7px; width: 10px;\">";
			echo "New&nbsp;command:";
			echo "</td><td>";
			echo "$cmd = $val";
			echo "</td>";

			echo "</tr>\n";
			echo "<tr><td style=\"padding-left: 7px; width: 10px;\">";
			echo "Existing&nbsp;command:";
			echo "</td><td style=\"border-top: 0px;\">";
			echo "$cmd = " . $webconfig_commands["$cmd"];
			echo "</td></tr>";
		} else {
			echo "<tr>";
			echo "<td style=\"padding-left: 44px; width: 10px;\">";
			echo "<input type='checkbox' name='" . $prefix ."_command[$cmd]' class=\"$prefix\" id=\"{$prefix}_command[$cmd]\"/></td>";
			echo "<td style=\"width: 99%\">$cmd = $val</td>";

			echo " </tr>\n";
		}
	}
	echo "</table></div>\n\n";
	echo "</td></tr>";
	echo "</table><br />";
}

if($import_avail == TRUE)
	$page->submit_button("Import selected commands", 'submit', false, array(
		'class' => 'button import-selected-commands',
		'onclick' => "return confirm('Continue importing and overwrite any matching checkcommands?\\nRemember to save your configuration when done.')",
	));

$page->form_end();
echo '</div>';
$page->html_end();
