<?php

/**
 * A contact definition is used to identify someone who should
 * be contacted in the event of a problem on your network. This is
 * not always a user, which means not all contacts can log into the
 * Monitor GUI.
 */
class contact_nacoma_Model extends naemon_nacoma_Model
{
	/** object type */
	const TYPE = 'contact';

	public $properties = array (
		'topic' => 'contact_name',
		'var_index' => array (
			'template' => 'select',
			'contact_name' => 'string',
			'alias' => 'string',
			'host_notifications_enabled' => 'bool',
			'service_notifications_enabled' => 'bool',
			'can_submit_commands' => 'bool',
			'retain_status_information' => 'bool',
			'retain_nonstatus_information' => 'bool',
			'host_notification_period' => 'select',
			'service_notification_period' => 'select',
			'host_notification_options' => 'checkbox',
			'service_notification_options' => 'checkbox',
			'host_notification_cmds' => 'select',
			'host_notification_cmds_args' => 'string',
			'service_notification_cmds' => 'select',
			'service_notification_cmds_args' => 'string',
			'contactgroups' => 'mselect',
			'register' => 'bool',
			'email' => 'string',
			'pager' => 'string',
			'address1' => 'string',
			'address2' => 'string',
			'address3' => 'string',
			'address4' => 'string',
			'address5' => 'string',
			'address6' => 'string',
			'file_id' => 'select'
		),
		'visible' => array (
			'template',
			'contact_name',
			'alias',
			'host_notifications_enabled',
			'service_notifications_enabled',
			'can_submit_commands',
			'retain_status_information',
			'retain_nonstatus_information',
			'host_notification_period',
			'service_notification_period',
			'host_notification_options',
			'service_notification_options',
			'host_notification_cmds',
			'host_notification_cmds_args',
			'service_notification_cmds',
			'service_notification_cmds_args',
			'contactgroups',
			'register',
			'email',
			'pager',
			'address1',
			'address2',
			'address3',
			'address4',
			'address5',
			'address6',
			'file_id'
		),
		'widget' => array (
			'register' => 'hidden'
		),
		'required' => array (
			'contact_name',
			'alias',
			'host_notification_period',
			'service_notification_period',
			'host_notification_options',
			'service_notification_options',
			'file_id'
		),
		'default' => array (
			'template' => 'default-contact',
			'service_notification_period' => '24x7',
			'host_notification_period' => '24x7',
			'service_notification_options' => array (
				'c',
				'w',
				'r'
			),
			'host_notification_options' => array (
				'd',
				'r'
			),
			'host_notifications_enabled' => 1,
			'service_notifications_enabled' => 1,
			'can_submit_commands' => 0,
			'retain_status_information' => 1,
			'retain_nonstatus_information' => 1,
			'service_notification_cmds' => 'service-notify',
			'host_notification_cmds' => 'host-notify',
			'file_id' => 'etc/contacts.cfg'
		),
		'special' => array (
			'host_notification_options' => array (
				'd' => 'Down',
				'u' => 'Unreachable',
				'r' => 'Recovery',
				'f' => 'Flapping start and stop',
				's' => 'Scheduled downtime start and stop',
				'n' => false
			),
			'service_notification_options' => array (
				'c' => 'Critical',
				'w' => 'Warning',
				'u' => 'Unknown',
				'r' => 'Recovery',
				'f' => 'Flapping start and stop',
				's' => 'Scheduled downtime start and stop',
				'n' => false
			),
			'file_id' => 'FILE',
			'template' => 'contact_template',
			'host_notification_period' => 'timeperiod',
			'service_notification_period' => 'timeperiod',
			'host_notification_cmds' => 'command',
			'service_notification_cmds' => 'command',
			'contactgroups' => 'contactgroup'
		),
		'label' => array (),
		'deprecated_names' => array ()
	);

	/**
	 * Return the mayi resource for the object
	 * @see object_nacoma_Model::mayi_resource()
	 */
	public function mayi_resource() {
		return "monitor.monitoring.contacts";
	}

	public function get_properties()
	{
		parent::get_properties();
		$this->properties['var_index']['enable_access'] = 'enable_access';
		$this->properties['default']['enable_access'] = false;
		$this->properties['visible'][] = 'enable_access';
		$user = new user_nacoma_Model();
		foreach($user->get_properties() as $property => $value) {
			if ($property === 'topic')
				continue;
			if(isset($value['username'])) {
				unset($value['username']);
			}

			$this->properties[$property] = array_merge($this->properties[$property], $value);
		}
		return $this->properties;
	}

	function save_object($user = true) {
		$real_props = $this->properties;
		$real_obj = $this->obj;
		$enable_access = isset($this->obj['enable_access']) && $this->obj['enable_access'];
		if (isset($this->obj['enable_access']))
			unset($this->obj['enable_access']);

		// remove the merged properties used for form generation
		$userpasswd = new user_nacoma_Model();
		foreach($userpasswd->get_properties() as $property => $value) {
			if ($property === 'topic')
				continue;
			foreach($value as $sub_property => $real_value) {
				unset($this->properties[$property][$sub_property]);
			}
		}

		if ($enable_access) {
			// ... and add the values the user gave us via the contact form
			foreach(array_keys($userpasswd->properties['var_index']) as $property) {
				if(!isset($this->obj[$property])) {
					// this if case is included because Username <> contact_name,
					// resolved outside foreach-loop
					continue;
				}
				$userpasswd->obj[$property] = $this->obj[$property];
				unset($this->obj[$property]);
			}
			if(isset($this->obj['contact_name']) && $this->obj['contact_name']) {
				$userpasswd->obj['username'] = $this->obj['contact_name'];
			}
		}
		else {
			foreach(array_keys($userpasswd->properties['var_index']) as $property) {
				if(isset($this->obj[$property])) {
					unset($this->obj[$property]);
				}
			}
			return parent::save_object($user);
		}

		if(!$this->validate_object()) {
			return false;
		}

		if(!$userpasswd->validate_object()) {
			$this->validation_error_list = $userpasswd->validation_error_list;
			return false;
		}

		if(!parent::save_object($user)) {
			return false;
		}

		$res = $userpasswd->save_object($user);

		$this->properties = $real_props;
		$this->obj = $real_obj;

		return $res;
	}

	/**
	 * Export with naming and values to map nagios struct names ond format
	 *
	 * Useful for Test this x operation
	 */
	public function get_nagios_struct($prefix='') {
		$this->expand_object();
		$obj = $this->denormalize();
		return array(
				/* FIXME */
				);
	}
}
