<?php
require_once('op5/mayi.php');

/**
 * A whole class dedicated only to that tiny thing at the top of the page. Wow!
 *
 * Sort of silly as a whole class, but we need to be able to add warning
 * messages during rendering, so it's better to have all the messages in one
 * place so they don't go out of sync.
 */
class top_bar_nacoma_Model {
	private static $unsaved_changes = 'You have unsaved configuration changes. Please press SAVE to verify and activate.';

	/**
	 * Render the top bar
	 */
	public static function render() {
		$to_show = array();
		$ret = false;

		$resources = array(
			'monitor.monitoring.hosts',
			'monitor.monitoring.services'
		);

		$env = array(
			'monitor' => array(
				'monitoring' => array(
					'hosts' => get_num_hosts_configured(),
					'services' => get_num_services_configured()
				)
			)
		);

		$mayi = op5MayI::instance();
		$exceeded = false;

		foreach ($resources as $resource) {
			if (!$mayi->run($resource . ':create.status', $env, $message)) {
				$exceeded = true;
			}
		}

		if ($exceeded) {
			$to_show[] = '<span id="too_many">You may not export further configuration, see the <a style="text-decoration: underline;" href="/monitor/op5/nacoma/export.php">save page</a> for further information.</span>';
		}

		if ((bool) Changelog_nacoma_Model::get_changes_count())
			$to_show[] = '<span id="unsaved">'.self::$unsaved_changes.'</span>';

		if (empty($to_show))
			return '<div class="config-warning"></div>';

		$ret .= '<div class="config-warning">';
		if (count($to_show) === 1)
			$ret .= '<p style="padding-top: 6px;">'.$to_show[0].'</p>';
		else
			$ret .= '<p>'.implode($to_show).'</p>';
		$ret .= '</div>';

		return $ret;
	}

	/**
	 * Add warning to the top-bar that there are changed but
	 * unsaved objects in the database
	 */
	public static function add_unsaved_warning() {
		if ((bool) Changelog_nacoma_Model::get_changes_count()) {
			?>
			<script type="text/javascript">
				$('.config-warning #unsaved').remove();
				if ($('.config-warning p').length == 0)
					$('.config-warning').append('<p></p>');

				$('.config-warning p').append('<span id="unsaved"><?php echo self::$unsaved_changes; ?></span>');
				if ($('.config-warning span').length > 1)
					$('.config-warning p').attr('style', '');
				else if ($('.config-warning span').length == 1)
					$('.config-warning p').attr('style', 'padding-top: 6px');
			</script>
			<?php
		}
	}

	/**
	 * Remove the unsaved changes warning from the top bar
	 */
	public static function remove_unsaved_warning() {
		?>
		<script type="text/javascript">
			$('.config-warning #unsaved').remove();
			if ($('.config-warning span').length == 0)
				$('.config-warning p').attr('style', 'padding: 0');
		</script>
		<?php
	}

	/**
	 * Adds warning to the top-bar with the supplied message.
	 */
	public static function add_warning($msg) {
		?>
		<script type="text/javascript">
			$('.config-warning #add_warning').remove();
			if ($('.config-warning p').length == 0) {
				$('.config-warning').append('<p></p>');
			}

			$('.config-warning p').append('<span id="add_warning"><?php echo $msg; ?></span>');
			if ($('.config-warning span').length > 1)
				$('.config-warning p').attr('style', '');
			else if ($('.config-warning span').length == 1)
				$('.config-warning p').attr('style', 'padding-top: 6px');
		</script>
		<?php
	}
}
